{% load cms_tags %}
{% page_attribute 'menu_title' 'shop-watch-list' as tooltip %}
<a href="{{ shop_watch_url }}" title="{{ tooltip }}">
	<i class="fa fa-heart fa-lg" uib-tooltip="{{ tooltip }}"></i>
</a>
